﻿<# 
.SYNOPSIS 
    Demo of SQL Always On Availability Group in Workgroup Cluster.
    
.DESCRIPTION 
    Defines virtual cluster nodes, prepares nodes and configures SQL Server Always ON Availability Group in workgroup cluster.
     
.NOTES 
    File Name  : WorkgroupCluster-Demo.ps1
    Author     : Ljubo Brodaric - ljubo@brodaric.com 
    Requires   : Windows Server 2019.
#>

### Do not run entire script
Exit


### Parameters and objects

$clusterName="c1-cluster"
$clusterIP="192.168.125.20"
$DNSSuffix = "lab125.local"

$adminPassword = "NTK19password!"
$localCredential = new-object -typename System.Management.Automation.PSCredential `
             -argumentlist "Administrator", (ConvertTo-SecureString $adminPassword -AsPlainText -Force)

Class DemoVM
    {
        [string]$VMName
        
    }

$VMs = @()

$VMs += (New-Object DemoVM -Property @{
          VMName = "c1-srv1"
          })

$VMs += (New-Object DemoVM -Property @{
          VMName = "c1-srv2"
          })

### Functions

function Wait-PSDirect
{
    # Waits until VM is ready to accept PowerShell Direct commands.

    param
    (
    [string] 
    $VMName, 
    
    $Cred
    )
    
    Write-LogMessage -SystemName $VMName -Message "Waiting for PowerShell Direct (using $($Cred.username))"
    # Trying to login to VM. Fully started VM generates login error.
    while ((Invoke-Command -VMName $VMName -Credential $Cred {"Test"} -ea SilentlyContinue) -ne "Test") {Sleep -Seconds 1}
}

function Reboot-VM
{
    param
    (
        [string]
        $VMName
    )
    
    Write-LogMessage $VMName "Rebooting"; Stop-VM $VMName; Start-VM $VMName
}

function Write-LogMessage
{
    # Function for displaying formatted log messages.  Also displays time in minutes since the script was started
    
    param
    (
        [string]
        $SystemName,
        
        [string]
        $Message
    )
    Write-Host (Get-Date).ToShortTimeString() -ForegroundColor Cyan -NoNewline;
    Write-Host " - [" -ForegroundColor White -NoNewline;
    Write-Host $SystemName -ForegroundColor Yellow -NoNewline;
    Write-Host "]::$($Message)" -ForegroundColor White;
}


### Create DNS Suffix

foreach ($vm in $vms) {
    Write-LogMessage -SystemName $VM.VMName -Message "Creating DNS Suffix"
    Invoke-Command -VMName $VM.VMName -Credential $localCredential {
        Set-ItemProperty "HKLM:\SYSTEM\CurrentControlSet\Services\Tcpip\Parameters\" -Name "NV Domain" -Value $using:DNSSuffix
        }
    Reboot-VM $VM.VMName; Wait-PSDirect -VMName $VM.VMName -Cred $localCredential
    Write-LogMessage -SystemName $VM.VMName -Message "VM rebooted and ready..."
}


### Add entries in hosts file

foreach ($vm in $vms) {
   Invoke-Command -VMName $VM.VMName -Credential $localCredential {
        Add-Content -Path "C:\WINDOWS\system32\drivers\etc\hosts" -Value "`n192.168.125.20    c1-cluster.lab125.local" -Force
        Add-Content -Path "C:\WINDOWS\system32\drivers\etc\hosts" -Value "`n192.168.125.21    c1-srv1.lab125.local" -Force
        Add-Content -Path "C:\WINDOWS\system32\drivers\etc\hosts" -Value "`n192.168.125.22    c1-srv2.lab125.local" -Force
        Add-Content -Path "C:\WINDOWS\system32\drivers\etc\hosts" -Value "`n192.168.125.23    c1-agl.lab125.local" -Force
        Add-Content -Path "C:\WINDOWS\system32\drivers\etc\hosts" -Value "`n192.168.125.5    DESKTOP-0BQIRRA" -Force
        }
}


### Open required firewall ports for SQL AG replication and SSMS

foreach ($vm in $vms) {
    Invoke-Command -VMName $VM.VMName -Credential $localCredential {
        New-NetFirewallRule -DisplayName "SQL Server" -Direction Inbound -LocalPort 1433 -Protocol TCP -Action Allow
        New-NetFirewallRule -DisplayName "SQL Server Browser" -Direction Inbound -LocalPort 1434 -Protocol UDP -Action Allow
        New-NetFirewallRule -DisplayName "SQL Server AG Replication" -Direction Inbound -LocalPort 5022 -Protocol TCP -Action Allow
        New-NetFirewallRule -DisplayName "ILB probe" -Direction Inbound -LocalPort 59999 -Protocol TCP -Action Allow
        }
}


### Enable non built-in administrators to create cluster

foreach ($vm in $vms) {
    Write-LogMessage -SystemName $VM.VMName -Message "Enabling non built-in administrators to create cluster"
    Invoke-Command -VMName $VM.VMName -Credential $localCredential {
        New-ItemProperty -path HKLM:\SOFTWARE\Microsoft\Windows\CurrentVersion\Policies\System -Name LocalAccountTokenFilterPolicy -Value 1
        }
}


### Create cluster

# Create an Array of node names
$nodeNames=@()
$nodeNames += foreach ($vm in $vms) {$VM.VMName}

# Create cluster
Invoke-Command -VMName c1-srv1 -Credential $localCredential {
    New-Cluster –Name $using:clusterName -Node $using:nodeNames -StaticAddress $using:clusterIP -AdministrativeAccessPoint DNS
    }

# Create SMB File Share Witness
# Execute on any node just once; username: DESKTOP\shareuser  password:NTK19password!

# Set-ClusterQuorum -FileShareWitness \\DESKTOP\FileShareWitness$ -Credential $(Get-Credential)



### Enable SQL Server Always ON

foreach ($vm in $vms) {
    $SQLInstance=$VM.VMName
    Write-LogMessage -SystemName $VM.VMName -Message "Enabling SQL Server Always ON and restarting SQL Instance"
    Invoke-Command -VMName $VM.VMName -Credential $localCredential {
        Enable-SqlAlwaysOn -ServerInstance $using:SQLInstance -Force
        }
}

